@@ -172,7 +172,7 @@ module Sass
     # @return [{Symbol => Object}] The normalized options hash.
     # @private
     def self.normalize_options(options)
-      options = DEFAULT_OPTIONS.merge(options.reject {|k, v| v.nil?})
+      options = DEFAULT_OPTIONS.merge(options.reject {|_k, v| v.nil?})
 
       # If the `:filename` option is passed in without an importer,
       # assume it's using the default filesystem importer.
@@ -192,6 +192,16 @@ module Sass
         options[:filesystem_importer].new(p.to_s)
       end
 
+      # Remove any deprecated importers if the location is imported explicitly
+      options[:load_paths].reject! do |importer|
+        importer.is_a?(Sass::Importers::DeprecatedPath) &&
+          options[:load_paths].find do |other_importer|
+            other_importer.is_a?(Sass::Importers::Filesystem) &&
+              other_importer != importer &&
+              other_importer.root == importer.root
+          end
+      end
+
       # Backwards compatibility
       options[:property_syntax] ||= options[:attribute_syntax]
       case options[:property_syntax]
@@ -219,7 +229,7 @@ module Sass
       had_syntax = options[:syntax]
 
       if had_syntax
-        # Use what was explicitly specificed
+        # Use what was explicitly specified
       elsif filename =~ /\.scss$/
         options.merge!(:syntax => :scss)
       elsif filename =~ /\.sass$/
@@ -466,7 +476,7 @@ ERR
 
         line_tabs = line_tab_str.scan(tab_str).size
         if tab_str * line_tabs != line_tab_str
-          message = <<END.strip.gsub("\n", ' ')
+          message = <<END.strip.tr("\n", ' ')
 Inconsistent indentation: #{Sass::Shared.human_indentation line_tab_str, true} used for indentation,
 but the rest of the document was indented using #{Sass::Shared.human_indentation tab_str}.
 END
@@ -479,15 +489,13 @@ END
     end
 
     # @comment
-    #   rubocop:disable ParameterLists
     def try_comment(line, last, tab_str, comment_tab_str, index)
-      # rubocop:enable ParameterLists
       return unless last && last.comment?
       # Nested comment stuff must be at least one whitespace char deeper
       # than the normal indentation
       return unless line =~ /^#{tab_str}\s/
       unless line =~ /^(?:#{comment_tab_str})(.*)$/
-        raise SyntaxError.new(<<MSG.strip.gsub("\n", " "), :line => index)
+        raise SyntaxError.new(<<MSG.strip.tr("\n", " "), :line => index)
 Inconsistent indentation:
 previous line was indented by #{Sass::Shared.human_indentation comment_tab_str},
 but this line was indented by #{Sass::Shared.human_indentation line[/^\s*/]}.
@@ -766,7 +774,19 @@ WARNING
                else
                  :normal
                end
-        Tree::CommentNode.new(value, type)
+        comment = Tree::CommentNode.new(value, type)
+        comment.line = line.index
+        text = line.text.rstrip
+        if text.include?("\n")
+          end_offset = text.length - text.rindex("\n")
+        else
+          end_offset = to_parser_offset(line.offset + text.length)
+        end
+        comment.source_range = Sass::Source::Range.new(
+          Sass::Source::Position.new(@line, to_parser_offset(line.offset)),
+          Sass::Source::Position.new(@line + text.count("\n"), end_offset),
+          @options[:filename])
+        comment
       else
         Tree::RuleNode.new(parse_interp(line.text), full_line_range(line))
       end
@@ -776,14 +796,12 @@ WARNING
       :each, :while, :if, :else, :extend, :import, :media, :charset, :content,
       :at_root, :error]
 
-    # @comment
-    #   rubocop:disable MethodLength
     def parse_directive(parent, line, root)
       directive, whitespace, value = line.text[1..-1].split(/(\s+)/, 2)
       raise SyntaxError.new("Invalid directive: '@'.") unless directive
       offset = directive.size + whitespace.size + 1 if whitespace
 
-      directive_name = directive.gsub('-', '_').to_sym
+      directive_name = directive.tr('-', '_').to_sym
       if DIRECTIVES.include?(directive_name)
         return send("parse_#{directive_name}_directive", parent, line, root, value, offset)
       end
@@ -838,8 +856,6 @@ WARNING
       )
       Tree::ExtendNode.new(interp_parsed, optional, selector_range)
     end
-    # @comment
-    #   rubocop:enable MethodLength
 
     def parse_warn_directive(parent, line, root, value, offset)
       raise SyntaxError.new("Invalid warn directive '@warn': expected expression.") unless value
@@ -1003,12 +1019,21 @@ WARNING
           end_pos = str.source_range.end_pos
           node = Tree::CssImportNode.new(str)
         else
-          media_parser = Sass::SCSS::Parser.new(scanner,
+          supports_parser = Sass::SCSS::Parser.new(scanner,
             @options[:filename], @options[:importer],
             @line, str.source_range.end_pos.offset)
-          media = media_parser.parse_media_query_list
-          end_pos = Sass::Source::Position.new(@line, media_parser.offset + 1)
-          node = Tree::CssImportNode.new(str, media.to_a)
+          supports_condition = supports_parser.parse_supports_clause
+
+          if scanner.eos?
+            node = Tree::CssImportNode.new(str, [], supports_condition)
+          else
+            media_parser = Sass::SCSS::Parser.new(scanner,
+              @options[:filename], @options[:importer],
+              @line, str.source_range.end_pos.offset)
+            media = media_parser.parse_media_query_list
+            end_pos = Sass::Source::Position.new(@line, media_parser.offset + 1)
+            node = Tree::CssImportNode.new(str, media.to_a, supports_condition)
+          end
         end
 
         node.source_range = Sass::Source::Range.new(
@@ -1170,10 +1195,15 @@ WARNING
           res << "\\" * (escapes - 1) << '#{'
         else
           res << "\\" * [0, escapes - 1].max
-          # Add 1 to emulate to_parser_offset.
-          res << Script::Parser.new(
-            scan, line, offset + scan.pos - scan.matched_size + 1, options).
-            parse_interpolated
+          if scan[1].include?("\n")
+            line = line + scan[1].count("\n")
+            offset = scan.matched_size - scan[1].rindex("\n")
+          else
+            offset += scan.matched_size
+          end
+          node = Script::Parser.new(scan, line, offset, options).parse_interpolated
+          offset = node.source_range.end_pos.offset
+          res << node
         end
       end
       res << rest
